######## Version 0.2 -- flexible design
##                      specifications  possible
#################################################
######## function for optimizing MA 2-col designs:
##################################################
######## Dirk.Repsilber@mpimp-golm.mpg.de
##################################################
## Parameter:
##
## c.matrix: Contrast matrix
## c.info:   Structure of contrast matrix
## print.every: Output of progress every X steps
##           of pair optimization loop
## number.units.of.exp: for a 2x8-design, this is
##           2*4=8
## index.already.compaired: giving the members of
##           all.pairs which are already hybridized
## number.total.arrays: all arrays available
##

##################################################
######## Preparation of default inputs:
##################################################
## preparing the contrast-matrix for a 2x4 design
## for estimating the 2-folded factor effects, E,
## and the interaction:
E.row   <- c(0,0,1,-1,1,-1,1,-1,1,-1)
GE.row1 <- c(0,0,1,-1,-1/3,1/3,-1/3,1/3,-1/3,1/3)
GE.row2 <- c(0,0,-1/3,1/3,1,-1,-1/3,1/3,-1/3,1/3)
GE.row3 <- c(0,0,-1/3,1/3,-1/3,1/3,1,-1,-1/3,1/3)
GE.row4 <- c(0,0,-1/3,1/3,-1/3,1/3,-1/3,1/3,1,-1)
cmat.E.GE  <- rbind(E.row,GE.row1,GE.row2,GE.row3,GE.row4)
colnames(cmat.E.GE) <- c("G","R","C1","C2","C3","C4","C5","C6","C7","C8")
rownames(cmat.E.GE) <- c("E","GxE","GxE","GxE","GxE")
cinfo.E.GE <- c(1,4)

## specification of already hybridized arrays as
## fixed part of the design for optimization:
old.design <- matrix(nrow=6,ncol=10,byrow=TRUE,
                     data=c(1,-1,0,-1,0,1,0,0,0,0,
                       1,-1,-1,0,1,0,0,0,0,0,
                       1,-1,0,0,-1,1,0,0,0,0,
                       1,-1,0,1,-1,0,0,0,0,0,
                       1,-1,1,-1,0,0,0,0,0,0,
                       1,-1,1,0,0,-1,0,0,0,0))

## initialization matrix 16, i.e. 10 additional arrays to be designed:
init.mat <- matrix(nrow=16,ncol=10,byrow=TRUE,
                   data=c(1,-1,0,-1,0,1,0,0,0,0,
                     1,-1,-1,0,1,0,0,0,0,0,
                     1,-1,0,0,-1,1,0,0,0,0,
                     1,-1,0,1,-1,0,0,0,0,0,
                     1,-1,1,-1,0,0,0,0,0,0,
                     1,-1,1,0,0,-1,0,0,0,0,
                     1,-1,0,0,0,0,0,-1,0,1,
                     1,-1,0,0,0,0,1,0,-1,0,
                     1,-1,0,0,0,-1,0,1,0,0,
                     1,-1,0,0,1,0,-1,0,0,0,
                     1,-1,0,-1,0,1,0,0,0,0,
                     1,-1,0,0,-1,0,0,1,0,0,
                     1,-1,0,0,0,-1,1,0,0,0,
                     1,-1,0,0,0,0,1,-1,0,0,
                     1,-1,0,0,0,0,0,-1,1,0,
                     1,-1,0,0,0,0,0,0,1,-1))

## initialization matrix 14, i.e. 8 additional arrays to be designed:
init.mat <- matrix(nrow=14,ncol=10,byrow=TRUE,
                   data=c(1,-1,0,-1,0,1,0,0,0,0,
                     1,-1,-1,0,1,0,0,0,0,0,
                     1,-1,0,0,-1,1,0,0,0,0,
                     1,-1,0,1,-1,0,0,0,0,0,
                     1,-1,1,-1,0,0,0,0,0,0,
                     1,-1,1,0,0,-1,0,0,0,0,
                     1,-1,0,0,0,0,0,-1,0,1,
                     1,-1,0,0,0,0,1,0,-1,0,
                     1,-1,0,0,0,-1,0,1,0,0,
                     1,-1,0,0,1,0,-1,0,0,0,
                     1,-1,0,-1,0,1,0,0,0,0,
                     1,-1,0,0,-1,0,0,1,0,0,
                     1,-1,0,0,0,-1,1,0,0,0,
                     1,-1,0,0,0,0,1,-1,0,0))


##################################################
######## Function for optimizing the MA design:
##################################################
optimized.MA.design <- function(c.matrix=cmat.E.GE,
                                c.info=cinfo.E.GE,
                                init=init.mat,
                                print.every=1000,
                                number.units.of.exp=8,
                                number.total.arrays=16,
                                fixed.part=old.design,
                                index.already.compaired=c(9,2,14,8,1,3)){
  require(daMA,quietly=TRUE)
  require(vsn,quietly=TRUE)

  ## function for generating designs from a given
  ## set-number:
  neue.zeile <- c(1,-1,rep(0,number.units.of.exp))
  generate.design <- function(set){
    design <- fixed.part
    for(i in 1:(number.total.arrays-length(index.already.compaired))){
      zeile <- neue.zeile
      zeile[2+set[1,i]] <- -1
      zeile[2+set[2,i]] <- 1
      design <- rbind(design,zeile)
    }
    rownames(design) <- c()
    design
  }
  
  ## initialization of optimization of sample pairs:
  best.design   <- init
  best.sets <- c()
  z <- 0
  all.pairs <- nchoosek(n=number.units.of.exp,k=2)
  all.pairs.for.optimization <- all.pairs[,-index.already.compaired] 
  n.all <- ncol(all.pairs.for.optimization)
  print(paste(n.all,"pairs are left for optimal choice with",number.total.arrays,"arrays!"))
  all.sets <- nchoosek(n=n.all,k=number.total.arrays-length(index.already.compaired)) 

  ## progress printing:
  print(paste("Working on pair optimization:"))
  print(paste("~",ncol(all.sets)/650000*2,"hours -- progress in percent:"))
  for(i in 1:100){
    a="."
    cat(a)
  }
  print("/")
  
  ## optimization loop for best sample pairs (88MB im RAM):
  for(set in 1:ncol(all.sets)){
    if(set%%round(ncol(all.sets)/100)==0) cat(".") # print a "." every percent progress
    this.set <- all.pairs.for.optimization[,all.sets[,set]]
    this.design <- generate.design(this.set)
    comp <- designMA(design.list=list(new=this.design,old=best.design),
                     cmat=c.matrix,
                     cinfo=c.info,
                     type='t')
    if(comp$effdesign=="new"){
      best.design <- this.design
      z <- z+1
      best.sets[z] <- set
      #print(paste("better:",set))
    }
  }
  print("done.")
  
  ## initialization for optimization of labelling:
  arrays.left <- number.total.arrays-length(index.already.compaired)
  lab <- matrix(nrow=arrays.left,ncol=2^arrays.left)
  for(arr in 1:arrays.left){
    ro <- unlist(rep(list(c(rep(1,2^(arrays.left-arr)),rep(2,2^(arrays.left-arr)))),(2^(arr-1))))
    lab[arr,] <- ro
  }

  generate.design.labelling <- function(set,labelling){
    design <- fixed.part
    labelling.scheme <- lab[,labelling]
    for(i in 1:arrays.left){
      zeile <- neue.zeile
      if(labelling.scheme[i]==1){
        zeile[2+set[1,i]] <- -1
        zeile[2+set[2,i]] <- 1
      } else {
        zeile[2+set[1,i]] <- 1
        zeile[2+set[2,i]] <- -1
      }
      design <- rbind(design,zeile)
    }
    rownames(design) <- c()
    design
  }
  best.labelled.design <- init
  best.labelled.sets   <- c()
  best.labelling       <- c()
  z <- 0

  ## optimization loop for optimization of labelling
  ## looping over best.sets only:
  for(set in best.sets){
    this.set <- all.pairs.for.optimization[,all.sets[,set]]
    for(labelling in 1:ncol(lab)){
      this.design <- generate.design.labelling(this.set,labelling)
      comp <- designMA(design.list=list(new=this.design,old=best.labelled.design),
                       cmat=c.matrix,
                       cinfo=c.info,
                       type='t')
      if(comp$effdesign=="new"){
        best.labelled.design <- this.design
        z <- z+1
        best.labelled.sets[z] <- set
        best.labelling[z] <- labelling
        print(paste("better:",set,labelling))
      }
    }
  }

  ## printed results output:
  designs <- list(old=init,new=best.design,newest=best.labelled.design)
  comp <- designMA(design.list=designs,
                   cmat=c.matrix,
                   cinfo=c.info,
                   type='t')
  print(comp)

  ## return:
  return(list(bd=best.design,
              bld=best.labelled.design,
              bs=best.sets,
              bls=best.labelled.sets))
}

output.design <- function(nodes = c("T1E1","T1E2","S1E1","S1E2","T2E1","T2E2","S2E1","S2E2"),
                          design = best.labelled.design,
                          filename.base = "out-design",
                          cols=c(rep('black',6),rep('red',10))){
  filename <- paste(filename.base,".sh",sep='')
  system(paste("cp out-kopf.txt",filename)) 
  for(z in 1:nrow(design)){
    start.pos <- (1:ncol(design))[design[z,]==1][2]
    ende.pos  <- (1:ncol(design))[design[z,]==-1][2]
    zeile <- paste(nodes[start.pos-2],"->",nodes[ende.pos-2],"[ color=",cols[z]," ];")
    write(zeile,file=filename,append=TRUE)
  }
  system(paste("cat out-ende.txt >>",filename))
  system(paste("chmod 777",filename))
  system(paste("./",filename,sep=''))
  system(paste("mkdir ",filename.base))
  system(paste("mv Optimal-design* ",filename.base,"/",sep=''))
  system(paste("mv ",filename," ",filename.base,"/",sep=''))
}



